=begin
PureCloud Platform API

With the PureCloud Platform API, you can control all aspects of your PureCloud environment. With the APIs you can access the system configuration, manage conversations and more.

OpenAPI spec version: v2
Contact: DeveloperEvangelists@genesys.com
Generated by: https://github.com/swagger-api/swagger-codegen.git

License: ININ
http://www.inin.com

Terms of Service: https://developer.mypurecloud.com/tos

=end

require 'date'

module PureCloud
  class ConversationQuery
    # Specifies the date and time range of data being queried. Results will include conversations that started, ended, or had any activity during the interval. Intervals are represented as an ISO-8601 string. For example: YYYY-MM-DDThh:mm:ss/YYYY-MM-DDThh:mm:ss
    attr_accessor :interval

    # Filters that target conversation-level data
    attr_accessor :conversation_filters

    # Filters that target quality management evaluation-level data
    attr_accessor :evaluation_filters

    # Filters that target individual segments within a conversation
    attr_accessor :segment_filters

    # Include faceted search and aggregate roll-ups describing your search results. This does not function as a filter, but rather, summary data about the data matching your filters
    attr_accessor :aggregations

    # Page size and number to control iterating through large result sets. Default page size is 25
    attr_accessor :paging

    # Sort the result set in ascending/descending order. Default is ascending
    attr_accessor :order

    # Specify which data element within the result set to use for sorting. The options  to use as a basis for sorting the results: conversationStart, segmentStart, and segmentEnd. If not specified, the default is conversationStart
    attr_accessor :order_by

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        
        :'interval' => :'interval',
        
        :'conversation_filters' => :'conversationFilters',
        
        :'evaluation_filters' => :'evaluationFilters',
        
        :'segment_filters' => :'segmentFilters',
        
        :'aggregations' => :'aggregations',
        
        :'paging' => :'paging',
        
        :'order' => :'order',
        
        :'order_by' => :'orderBy'
        
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        
        :'interval' => :'String',
        
        :'conversation_filters' => :'Array<AnalyticsQueryFilter>',
        
        :'evaluation_filters' => :'Array<AnalyticsQueryFilter>',
        
        :'segment_filters' => :'Array<AnalyticsQueryFilter>',
        
        :'aggregations' => :'Array<AnalyticsQueryAggregation>',
        
        :'paging' => :'PagingSpec',
        
        :'order' => :'String',
        
        :'order_by' => :'String'
        
      }
    end

    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}){|(k,v), h| h[k.to_sym] = v}

      
      if attributes.has_key?(:'interval')
        
        
        self.interval = attributes[:'interval']
        
      
      end

      
      if attributes.has_key?(:'conversationFilters')
        
        if (value = attributes[:'conversationFilters']).is_a?(Array)
          self.conversation_filters = value
        end
        
        
      
      end

      
      if attributes.has_key?(:'evaluationFilters')
        
        if (value = attributes[:'evaluationFilters']).is_a?(Array)
          self.evaluation_filters = value
        end
        
        
      
      end

      
      if attributes.has_key?(:'segmentFilters')
        
        if (value = attributes[:'segmentFilters']).is_a?(Array)
          self.segment_filters = value
        end
        
        
      
      end

      
      if attributes.has_key?(:'aggregations')
        
        if (value = attributes[:'aggregations']).is_a?(Array)
          self.aggregations = value
        end
        
        
      
      end

      
      if attributes.has_key?(:'paging')
        
        
        self.paging = attributes[:'paging']
        
      
      end

      
      if attributes.has_key?(:'order')
        
        
        self.order = attributes[:'order']
        
      
      end

      
      if attributes.has_key?(:'orderBy')
        
        
        self.order_by = attributes[:'orderBy']
        
      
      end

      
    end

    # Show invalid properties with the reasons. Usually used together with valid?
    # @return Array for valid properies with the reasons
    def list_invalid_properties
      invalid_properties = Array.new
      
      
      return invalid_properties
    end

    # Check to see if the all the properties in the model are valid
    # @return true if the model is valid
    def valid?
      
      
      
      
      
      
      
      
      
      
      
      
      
      
      
      
      
      
      
      
      
      
      
      
      
      
      
      allowed_values = ["asc", "desc"]
      if @order && !allowed_values.include?(@order)
        return false
      end
      
      
      
      
      
      allowed_values = ["conversationStart", "conversationEnd", "segmentStart", "segmentEnd"]
      if @order_by && !allowed_values.include?(@order_by)
        return false
      end
      
      
      
    end

    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] order Object to be assigned
    def order=(order)
      allowed_values = ["asc", "desc"]
      if order && !allowed_values.include?(order)
        fail ArgumentError, "invalid value for 'order', must be one of #{allowed_values}."
      end
      @order = order
    end

    
    
    
    
    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] order_by Object to be assigned
    def order_by=(order_by)
      allowed_values = ["conversationStart", "conversationEnd", "segmentStart", "segmentEnd"]
      if order_by && !allowed_values.include?(order_by)
        fail ArgumentError, "invalid value for 'order_by', must be one of #{allowed_values}."
      end
      @order_by = order_by
    end

    
    
    
    # Checks equality by comparing each attribute.
    # @param [Object] Object to be compared
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          interval == o.interval &&
          conversation_filters == o.conversation_filters &&
          evaluation_filters == o.evaluation_filters &&
          segment_filters == o.segment_filters &&
          aggregations == o.aggregations &&
          paging == o.paging &&
          order == o.order &&
          order_by == o.order_by
    end

    # @see the `==` method
    # @param [Object] Object to be compared
    def eql?(o)
      self == o
    end

    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [interval, conversation_filters, evaluation_filters, segment_filters, aggregations, paging, order, order_by].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /^(true|t|yes|y|1)$/i
          true
        else
          false
        end
      when :Object
        # generic object (usually a Hash), return directly
        value
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = Object.const_get("PureCloud").const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end

  end
end
